-- ********************************************************************************
-- Broker Statistics
-- A Data Broker display for combat statistics.
-- By: Shenton
--
-- Config.lua
-- Ace3 configuration panel and databases.
-- ********************************************************************************

local A = _G["DataBrokerStatisticsGlobal"];
local L = LibStub("AceLocale-3.0"):GetLocale("BrokerStats");

-- Lua globals
local format, pairs, smatch = format, pairs, string.match;

-- ********************************************************************************
-- Configuration
-- ********************************************************************************

--- Default database table
local db = {};
function A:AceDefaultDB()
    db =
    {
        profile =
        {
            verbosity = 1,
            time = 1,
            displayPre = 1,
            displayPost = 1,
            shareWhisper = 1,
            shareGuild = 1,
            shareRaid = 1,
            shareBg = 1,
            buildInStats =
            {
                primary = {},
                secondary = {}
            },
            custom =
            {
                primary = {},
                secondary = {}
            },
            dataBroker =
            {
                customDiplays = {}
            }
        },
        global =
        {
            custom = {}
        }
    };

    return db;
end

--- AceConfig menu
local options, addName, deleteCustom, deleteDbObject, addString = {}, nil, {}, {}, nil;
function A:AceConfig()
    options =
    {
        name = L["Broker Statistics"],
        type = "group",
        childGroups = "tab",
        args =
        {
            options =
            {
                order = 0,
                name = L["Options"],
                type = "group",
                args =
                {
                    main =
                    {
                        order = 0,
                        name = L["Main"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            verbosity =
                            {
                                order = 0,
                                name = L["Verbosity"],
                                desc = L["Set the addon chat messages verbosity."],
                                type = "select",
                                values = { [0] = L["Disabled"], [1] = L["Normal"], [2] = L["Everything"] },
                                set = function(info, val) A.db.profile.verbosity = val; end,
                                get = function(info) return A.db.profile.verbosity; end
                            },
                            refreshRate =
                            {
                                order = 1,
                                name = L["Refresh Rate"],
                                desc = L["The refresh rate in seconds."],
                                type = "range",
                                min = 0.1,
                                max = 5,
                                step = 0.1,
                                set = function(info, val)
                                    A.db.profile.time = val;
                                    A:RefreshTimer();
                                end,
                                get = function(info) return A.db.profile.time; end
                            }
                        }
                    },
                    display =
                    {
                        order = 1,
                        name = L["Display"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            pre =
                            {
                                order = 0,
                                name = L["Pre"],
                                desc = L["Display the stat prefix."],
                                type = "toggle",
                                set = function(info, val) A.db.profile.displayPre = val; end,
                                get = function(info) return A.db.profile.displayPre; end
                            },
                            post =
                            {
                                order = 1,
                                name = L["Post"],
                                desc = L["Display the stat post string."],
                                type = "toggle",
                                set = function(info, val) A.db.profile.displayPost = val; end,
                                get = function(info) return A.db.profile.displayPost; end
                            },
                        }
                    },
                    dataBroker =
                    {
                        order = 2,
                        name = L["Data Broker"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            addNew =
                            {
                                order = 0,
                                name = L["Add New"],
                                type = "group",
                                inline = true,
                                args =
                                {
                                    inputAdd =
                                    {
                                        order = 0,
                                        name = L["Enter Name"],
                                        desc = L["Enter the new Data Broker object name."],
                                        type = "input",
                                        set = function(info, val) addName = val; end,
                                        get = function() return addName; end
                                    },
                                    executeAdd =
                                    {
                                        order = 1,
                                        name = L["Create"],
                                        disabled = function() return not addName; end,
                                        desc = L["Create the new Data Broker object %s"]:format(addName or ""),
                                        type = "execute",
                                        func = function(info, val)
                                            addName = strtrim(addName);
                                            if ( not addName or addName == "" ) then
                                                A:Message(L["The name provided was empty."], 1, 1);
                                                return;
                                            end

                                            if ( A.db.profile.dataBroker.customDiplays[addName] ) then
                                                A:Message(L["The Data Broker object %s already exists."]:format(addName), 1, 1);
                                                addName = nil;
                                            else
                                                A.db.profile.dataBroker.customDiplays[addName] =
                                                {
                                                    enabled = 1
                                                };
                                                addName = nil;
                                                A:RefreshConfig();
                                            end
                                        end
                                    }
                                }
                            }
                        }
                    },
                    shareReject =
                    {
                        order = 3,
                        name = L["Reject share request"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            whisper =
                            {
                                order = 0,
                                name = L["Player"],
                                desc = L["Reject share from other players."],
                                type = "toggle",
                                get = function() return A.db.profile.shareWhisper; end,
                                set = function(info, val) A.db.profile.shareWhisper = val; end
                            },
                            guild =
                            {
                                order = 1,
                                name = L["Guild"],
                                desc = L["Reject share from guild."],
                                type = "toggle",
                                get = function() return A.db.profile.shareGuild; end,
                                set = function(info, val) A.db.profile.shareGuild = val; end
                            },
                            raid =
                            {
                                order = 2,
                                name = L["Raid"],
                                desc = L["Reject share from raid or party."],
                                type = "toggle",
                                get = function() return A.db.profile.shareRaid; end,
                                set = function(info, val) A.db.profile.shareRaid = val; end
                            },
                            bg =
                            {
                                order = 3,
                                name = L["Battleground"],
                                desc = L["Reject share from battleground."],
                                type = "toggle",
                                get = function() return A.db.profile.shareBg; end,
                                set = function(info, val) A.db.profile.shareBg = val; end
                            }
                        }
                    }
                }
            },
            buildInStats =
            {
                order = 1,
                name = L["Build in Stats"],
                type = "group",
                args =
                {
                    misc =
                    {
                        order = 0,
                        name = L["Miscellaneous"],
                        type = "group",
                        inline = true,
                        args = {}
                    },
                    tank =
                    {
                        order = 1,
                        name = L["Tank"],
                        type = "group",
                        inline = true,
                        args = {}
                    },
                    melee =
                    {
                        order = 2,
                        name = L["Melee"],
                        type = "group",
                        inline = true,
                        args = {}
                    },
                    ranged =
                    {
                        order = 3,
                        name = L["Ranged"],
                        type = "group",
                        inline = true,
                        args = {}
                    },
                    spell =
                    {
                        order = 4,
                        name = L["Spell"],
                        type = "group",
                        inline = true,
                        args = {}
                    }
                }
            },
            customStats =
            {
                order = 2,
                name = L["Custom Stats"],
                type = "group",
                args =
                {
                    addNew =
                    {
                        order = 0,
                        name = L["Add New"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            header1 =
                            {
                                order = 0,
                                name = L["Create New"],
                                type = "header"
                            },
                            inputAdd =
                            {
                                order = 1,
                                name = L["Enter Name"],
                                desc = L["Enter the new statistic name."],
                                type = "input",
                                set = function(info, val) addName = val; end,
                                get = function() return addName; end
                            },
                            executeAdd =
                            {
                                order = 2,
                                name = L["Create"],
                                disabled = function() return not addName; end,
                                desc = L["Create the new custom statistic %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    addName = strtrim(addName);
                                    if ( not addName or addName == "" ) then
                                        A:Message(L["The name provided was empty."], 1, 1);
                                        return;
                                    end

                                    if ( A.db.global.custom[addName] ) then
                                        A:Message(L["The custom stat %s already exists."]:format(addName), 1, 1);
                                        addName = nil;
                                    else
                                        A.db.global.custom[addName] = {};
                                        addName = nil;
                                    end
                                end
                            },
                            header10 =
                            {
                                order = 10,
                                name = L["Import from string"],
                                type = "header"
                            },
                            inputAddString =
                            {
                                order = 11,
                                name = L["Paste String"],
                                desc = L["Paste here the new statistic string."],
                                type = "input",
                                multiline = 1,
                                width = "full",
                                set = function(info, val) addString = val; end,
                                get = function() return addString; end
                            },
                            executeAddString =
                            {
                                order = 12,
                                name = L["Create"],
                                disabled = function() return not addString; end,
                                desc = L["Create the new custom statistic %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    local customName = smatch(addString, "^(.+){");
                                    local customText = smatch(addString, "{(.+)}");
                                    customText = "{"..customText.."}";

                                    if ( A.db.global.custom[customName] ) then
                                        local pop = StaticPopup_Show("BrokerStatsAddCustomNameExists");
                                        if (pop) then pop.customText = customText; end
                                        addString = nil;
                                    else
                                        local customTable = loadstring("return "..customText);
                                        customTable = customTable();

                                        if ( customName ) then
                                            A.db.global.custom[customName] = customTable;
                                            LibStub("AceConfigRegistry-3.0"):NotifyChange("BrokerStatsConfig");
                                            addString = nil;
                                        else
                                            A:Message(L["The name provided was empty."], 1, 1);
                                        end
                                    end
                                end
                            },
                        }
                    }
                }
            }
        }
    };

    if ( A:GotCustomLdb() ) then
        options.args.options.args.dataBroker.args.list =
        {
            order = 1,
            name = L["List"],
            type = "group",
            inline = true,
            args = {}
        };
    end

    local order = 0;
    for k,v in A:PairsByKeys(A.db.profile.dataBroker.customDiplays) do
        options.args.options.args.dataBroker.args.list.args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                enabled =
                {
                    order = 0,
                    name = L["Enable"],
                    desc = L["Enable this Data Broker object.\n\nDisabling a Data Broker object require reloading the user interface."],
                    confirm = function()
                        if ( A.db.profile.dataBroker.customDiplays[k].enabled ) then
                            return L["Disable %s Data Broker object and reload the user interface?"]:format(k);
                        end
                        return nil;
                    end,
                    type = "toggle",
                    set = function(info, val)
                        A.db.profile.dataBroker.customDiplays[k].enabled = val;
                        if ( not val ) then ReloadUI(); end
                        A:RefreshConfig();
                    end,
                    get = function(info) return A.db.profile.dataBroker.customDiplays[k].enabled; end
                },
                lineBreak1 =
                {
                    order = 1,
                    name = "",
                    type = "description",
                    width = "full"
                },
                deleteToggle =
                {
                    order = 2,
                    name = L["Delete"],
                    desc = L["Activate the delete button."],
                    type = "toggle",
                    set = function(info, val) deleteDbObject[k] = val; end,
                    get = function(info) return deleteDbObject[k]; end
                },
                deleteExecute =
                {
                    order = 3,
                    name = L["Delete"],
                    desc = L["Delete the %s Data Broker object.\n\nThis will require reloading the user interface.\n\n|cffff3333No more warning is given beyond this point."]:format(k),
                    disabled = function() return not deleteDbObject[k]; end,
                    type = "execute",
                    func = function() 
                        A.db.profile.dataBroker.customDiplays[k] = nil;
                        ReloadUI();
                    end
                }
            }
        };
        order = order + 1;
    end

    order = 0;
    local dbObjectsSelect = A:CreateDbObjectsSelect();
    for k,v in A:PairsByKeys(A.buildInStats) do
        options.args.buildInStats.args[v.type].args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                enable =
                {
                    order = 0,
                    name = L["Enable"],
                    desc = L["Enable %s on the data broker display.\n\nCalculation:\n|cff006666%s"]:format(k, v.calc),
                    type = "toggle",
                    set = function(info, val)
                        if ( val ) then
                            if ( not A.db.profile.buildInStats[A.playerSpec][k] ) then
                                A.db.profile.buildInStats[A.playerSpec][k] =
                                {
                                    enabled = 1
                                };
                            end
                        else
                            A.db.profile.buildInStats[A.playerSpec][k] = nil;
                        end
                    end,
                    get = function(info)
                        if ( not A.db.profile.buildInStats[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.buildInStats[A.playerSpec][k].enabled;
                    end
                },
                dataBrokerObject =
                {
                    order = 1,
                    name = L["Data Broker Object"],
                    desc = L["Choose on which Data Broker object to display %s."]:format(k),
                    type = "select",
                    values = dbObjectsSelect,
                    disabled = function()
                        if ( not A.db.profile.buildInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.buildInStats[A.playerSpec][k].enabled;
                    end,
                    set = function(info, val) A.db.profile.buildInStats[A.playerSpec][k].dbObject = val; end,
                    get = function(info)
                        if ( A.db.profile.buildInStats[A.playerSpec][k] and A.db.profile.buildInStats[A.playerSpec][k].dbObject ) then
                            return A.db.profile.buildInStats[A.playerSpec][k].dbObject;
                        else
                            return L["Default"];
                        end
                    end
                },
            }
        };
        order = order + 1;
    end

    order = 1;
    for k,v in A:PairsByKeys(A.db.global.custom) do
        local calc = v.calc or L["Not Available"];

        options.args.customStats.args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                header1 =
                {
                    order = 0,
                    name = L["Main"],
                    type = "header"
                },
                enabled =
                {
                    order = 1,
                    name = L["Enable"],
                    desc = L["Enable %s on the data broker display.\n\nCalculation:\n|cff339999%s"]:format(k, calc),
                    type = "toggle",
                    set = function(info, val)
                        if ( val ) then
                            if ( not A.db.profile.custom[A.playerSpec][k] ) then
                                A.db.profile.custom[A.playerSpec][k] =
                                {
                                    enabled = 1
                                };
                            end
                        else
                            A.db.profile.custom[A.playerSpec][k] = nil;
                        end
                    end,
                    get = function(info)
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.custom[A.playerSpec][k].enabled;
                    end
                },
                dataBrokerObject =
                {
                    order = 2,
                    name = L["Data Broker Object"],
                    desc = L["Choose on which Data Broker object to display %s."]:format(k),
                    type = "select",
                    values = dbObjectsSelect,
                    disabled = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.custom[A.playerSpec][k].enabled;
                    end,
                    set = function(info, val) A.db.profile.custom[A.playerSpec][k].dbObject = val; end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k] and A.db.profile.custom[A.playerSpec][k].dbObject ) then
                            return A.db.profile.custom[A.playerSpec][k].dbObject;
                        else
                            return L["Default"];
                        end
                    end
                },
                lineBreak1 =
                {
                    order = 3,
                    name = "",
                    type = "description",
                    width = "full"
                },
                deleteToggle =
                {
                    order = 4,
                    name = L["Delete"],
                    desc = L["Activate the delete button."],
                    type = "toggle",
                    set = function(info, val) deleteCustom[k] = val; end,
                    get = function(info) return deleteCustom[k]; end
                },
                deleteExecute =
                {
                    order = 5,
                    name = L["Delete"],
                    desc = L["Delete the %s custom statistic.\n|cffff3333No more warning is given beyond this point."]:format(k),
                    disabled = function() return not deleteCustom[k]; end,
                    type = "execute",
                    func = function()
                        A.db.global.custom[k] = nil;
                        A.db.profile.custom["primary"][k] = nil;
                        A.db.profile.custom["secondary"][k] = nil;
                    end
                },
                header2 =
                {
                    order = 10,
                    name = L["Options"],
                    type = "header"
                },
                pre =
                {
                    order = 11,
                    name = L["Pre"],
                    desc = L["The text to display before the statistic, ex: \"AP: \"."],
                    type = "input",
                    set = function(info, val) A.db.global.custom[k].pre = val; end,
                    get = function(info, val) return A.db.global.custom[k].pre; end
                },
                post =
                {
                    order = 12,
                    name = L["Post"],
                    desc = L["The text to display after the statistic, ex: \"%\"\nNote: this is displayed after any stat in multiple mode."],
                    type = "input",
                    set = function(info, val) A.db.global.custom[k].post = val; end,
                    get = function(info, val) return A.db.global.custom[k].post; end
                },
                lineBreak20 =
                {
                    order = 20,
                    name = "",
                    type = "description",
                    width = "full"
                },
                multiple =
                {
                    order = 21,
                    name = L["Multiple"],
                    desc = L["Tell the display engine there is more than one statistic returned."],
                    type = "toggle",
                    disabled = function() return not A.db.global.custom[k].separator; end,
                    set = function(info, val) A.db.global.custom[k].multiple = val; end,
                    get = function(info) return A.db.global.custom[k].multiple; end
                },
                separator =
                {
                    order = 22,
                    name = L["Separator"],
                    desc = L["The separator used in multiple mode."],
                    type = "input",
                    set = function(info, val)
                        if ( val == "" ) then
                            val = nil;
                            A.db.global.custom[k].multiple = nil;
                        end
                        A.db.global.custom[k].separator = val; end,
                    get = function(info, val) return A.db.global.custom[k].separator; end
                },
                lineBreak21 =
                {
                    order = 23,
                    name = "",
                    type = "description",
                    width = "full"
                },
                round =
                {
                    order = 24,
                    name = L["Round"],
                    desc = L["Round the result to the specified decimal places."],
                    type = "select",
                    values = { [1000] = L["Disabled"], [0] = 0, [1] = 1, [2] = 2, [3] = 3, [4] = 4, [5] = 5 },
                    set = function(info, val)
                        if ( val == 1000 ) then
                            A.db.global.custom[k].round = nil;
                        else
                            A.db.global.custom[k].round = val;
                        end
                    end,
                    get = function(info)
                        if ( A.db.global.custom[k].round ) then
                            return A.db.global.custom[k].round;
                        else
                            return 1000;
                        end
                    end
                },
                calc =
                {
                    order = 25,
                    name = L["Calculation Code"],
                    desc = L["The code used to calculate the statistic."],
                    type = "input",
                    multiline = 1,
                    width = "full",
                    set = function(info, val) A.db.global.custom[k].calc = val; end,
                    get = function(info, val) return A.db.global.custom[k].calc; end
                },
                header3 =
                {
                    order = 30,
                    name = L["Share"],
                    type = "header"
                },
                sharePlayer =
                {
                    order = 31,
                    name = L["Player"],
                    desc = L["Send %s to another player."]:format(k),
                    type = "execute",
                    func = function()
                        local pop = StaticPopup_Show("BrokerStatsSendCustom");
                        if (pop) then pop.customStat = k; end
                    end
                },
                shareGuild =
                {
                    order = 32,
                    name = L["Guild"],
                    desc = L["Send %s to your guild."]:format(k),
                    type = "execute",
                    func = function() A:SendStat("GUILD", k); end
                },
                shareRaid =
                {
                    order = 33,
                    name = L["Raid"],
                    desc = L["Send %s to your raid. It will send it to your group if you are in a party."]:format(k),
                    type = "execute",
                    func = function() A:SendStat("RAID", k); end
                },
                shareBg =
                {
                    order = 34,
                    name = L["Battleground"],
                    desc = L["Send %s to your battleground raid."]:format(k),
                    type = "execute",
                    func = function() A:SendStat("BATTLEGROUND", k); end
                },
                shareInput =
                {
                    order = 40,
                    name = L["String"],
                    desc = L["Use this to store this custom stat."],
                    type = "input",
                    multiline = 1,
                    width = "full",
                    get = function() return k..A:TableTostring(v); end,
                    set = false
                }
            }
        };
        order = order + 1;
    end

    -- Profiles
    options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(A.db);
    options.args.profile.order = 1000;

    return options;
end
